import os

import cerberus
from cerberus.solvers import Solver, CodeInput

import numpy as np

cerberus.LOG.set_verbosity(1)

# --- Initialize solvers

sss_input = CodeInput("./Serpent.inp")
sss_path = os.environ.get("SERPENT_EXE_PATH")
sss = Solver("Serpent", sss_path, ["-omp", "8", "-port"])
sss.input = sss_input
sss.initialize()

# --- Get ZAI list from some material

tra = sss.get_output_variable("sss_ov_composition_fuelz1_zai")
tra.communicate()

print(tra.value_vec)

# --- Get handle to detector transferrable

det_tra = sss.get_output_variable("sss_ov_DET_fission")

# --- Do a couple of iterations

nz = 10
n_iter = 5

for cur_iter in range(n_iter):

    # --- Run Serpent calculation

    sss.solve()

    # --- Get new values from Serpent

    det_tra.communicate()

    # --- Get relative fission rates

    values = det_tra.value_vec.copy()
    rel_values = values/np.mean(values)

    # --- Invert these

    inv_rel_values = 1/rel_values

    # --- Adjust nuclide densities

    adenses = np.zeros((nz,))
    for k  in range(nz):
        out_adens_tra = sss.get_output_variable(f"sss_ov_composition_fuelz{k+1}_adens")
        out_adens_tra.communicate()

        cur_values = out_adens_tra.value_vec.copy()

        u235_idx = 0
        delta_adens = 4e-5

        # --- Store atomic density corresponding to previous power distribution

        adenses[k] = cur_values[u235_idx]

        cur_values[u235_idx] = cur_values[u235_idx] + delta_adens*(inv_rel_values[k] - 1)

        # --- Set and send values

        in_adens_tra = sss.get_input_variable(f"sss_iv_composition_fuelz{k+1}_adens")
        in_adens_tra.set_and_send(cur_values)

    # --- Print something

    print(f"\nIteration {cur_iter+1}")
    print(f"{'axial zone':10s}  {'relative power':15s}  {'U-235 content':15s} ")
    for k in range(nz):
        print(f"{10-k:10d} {rel_values[-1-k]:15.2f} {adenses[-1-k]:15.5E}")